"use strict";

var _react = _interopRequireDefault(require("react"));

var _chai = require("chai");

var _react2 = require("@testing-library/react");

var _Draggable = require("./Draggable");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _objectWithoutProperties(source, excluded) { if (source == null) return {}; var target = _objectWithoutPropertiesLoose(source, excluded); var key, i; if (Object.getOwnPropertySymbols) { var sourceSymbolKeys = Object.getOwnPropertySymbols(source); for (i = 0; i < sourceSymbolKeys.length; i++) { key = sourceSymbolKeys[i]; if (excluded.indexOf(key) >= 0) continue; if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue; target[key] = source[key]; } } return target; }

function _objectWithoutPropertiesLoose(source, excluded) { if (source == null) return {}; var target = {}; var sourceKeys = Object.keys(source); var key, i; for (i = 0; i < sourceKeys.length; i++) { key = sourceKeys[i]; if (excluded.indexOf(key) >= 0) continue; target[key] = source[key]; } return target; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function ownKeys(object, enumerableOnly) { var keys = Object.keys(object); if (Object.getOwnPropertySymbols) { var symbols = Object.getOwnPropertySymbols(object); if (enumerableOnly) symbols = symbols.filter(function (sym) { return Object.getOwnPropertyDescriptor(object, sym).enumerable; }); keys.push.apply(keys, symbols); } return keys; }

function _objectSpread(target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i] != null ? arguments[i] : {}; if (i % 2) { ownKeys(source, true).forEach(function (key) { _defineProperty(target, key, source[key]); }); } else if (Object.getOwnPropertyDescriptors) { Object.defineProperties(target, Object.getOwnPropertyDescriptors(source)); } else { ownKeys(source).forEach(function (key) { Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key)); }); } } return target; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

describe('draggable', function () {
  var defaultStyle = {
    position: 'fixed',
    top: '11px',
    left: '11px'
  };
  var utils;
  beforeEach(function () {
    utils = setup();
  });
  afterEach(_react2.cleanup);
  it('should supply proper props to target', function () {
    (0, _chai.expect)(utils.container.querySelector('[aria-grabbed]')).not.to.be.ok;
  });
  describe('starting drag', function () {
    it('should supply proper props to target', function () {
      var _utils = utils,
          getByText = _utils.getByText,
          container = _utils.container;

      _react2.fireEvent.mouseDown(getByText(/handle/));

      (0, _chai.expect)(container.querySelector('[aria-grabbed]')).to.be.ok;
    });
    it('should return a delta position', function () {
      var startAt = {
        clientX: 10,
        clientY: 10
      };
      var delta = {
        x: 5,
        y: 5
      };
      var _utils2 = utils,
          drag = _utils2.drag,
          getByText = _utils2.getByText;
      drag({
        start: startAt,
        delta: delta
      });
      (0, _chai.expect)(getByText("".concat(delta.x, ", ").concat(delta.y))).to.be.ok;
    });
    it('should return a correct delta position after more drag', function () {
      var startAt = {
        clientX: 10,
        clientY: 10
      };
      var delta = {
        x: 5,
        y: 5
      };
      var secondStart = {
        clientX: startAt.clientX + delta.x,
        clientY: startAt.clientY + delta.y
      };
      var _utils3 = utils,
          drag = _utils3.drag,
          getByText = _utils3.getByText;
      drag({
        start: startAt,
        delta: delta
      });
      drag({
        start: secondStart,
        delta: delta
      });
      (0, _chai.expect)(getByText("".concat(2 * delta.x, ", ").concat(2 * delta.y))).to.be.ok;
    });
    describe('decorate with styles', function () {
      beforeEach(function () {
        (0, _react2.cleanup)();
        utils = setup({
          controlStyle: true
        });
      });
      it("should change target's style when dragging", function () {
        var _utils4 = utils,
            getByTestId = _utils4.getByTestId,
            drag = _utils4.drag;
        drag({
          start: {
            clientX: 3,
            clientY: 3
          },
          delta: {
            x: 10,
            y: 15
          }
        });
        (0, _chai.expect)(getByTestId('main').style.transform).to.equal("translate(10px, 15px)");
      });
      it("should change target's style when dragging (touches)", function () {
        var _utils5 = utils,
            getByTestId = _utils5.getByTestId,
            drag = _utils5.drag;
        drag({
          start: {
            clientX: 3,
            clientY: 3
          },
          delta: {
            x: 10,
            y: 15
          },
          touch: true
        });
        (0, _chai.expect)(getByTestId('main').style.transform).to.equal("translate(10px, 15px)");
      });
      it('should set proper cursor', function () {
        (0, _chai.expect)(utils.getByText(/handle/i).style.cursor).to.equal('grab');
      });
      it('should change cursor while dragging', function () {
        var _utils6 = utils,
            getByText = _utils6.getByText;

        _react2.fireEvent.mouseDown(getByText(/handle/i));

        (0, _chai.expect)(getByText(/handle/i).style.cursor).to.equal('grabbing');
      });
      it('should add `will-change: transform` to target', function () {
        var _utils7 = utils,
            getByText = _utils7.getByText,
            getByTestId = _utils7.getByTestId;

        _react2.fireEvent.mouseDown(getByText(/handle/));

        (0, _chai.expect)(getByTestId('main').style.willChange).to.equal('transform');
      });
      it('should remove `will-change: transform` from target', function () {
        var _utils8 = utils,
            getByTestId = _utils8.getByTestId,
            drag = _utils8.drag;
        drag({
          start: {
            clientX: 3,
            clientY: 3
          },
          delta: {
            x: 10,
            y: 15
          }
        });
        (0, _chai.expect)(getByTestId('main').style.willChange).to.equal('');
      });
    });
    describe('ending drag', function () {
      it('should supply proper props to target', function () {
        var _utils9 = utils,
            drag = _utils9.drag,
            container = _utils9.container;
        drag();
        (0, _chai.expect)(container.querySelector('[aria-grabbed]')).not.to.be.ok;
      });
    });
    describe('limit in viewport', function () {
      beforeEach(function () {
        (0, _react2.cleanup)();
        utils = setup({
          controlStyle: true,
          viewport: true,
          style: _objectSpread({}, defaultStyle, {
            width: '180px',
            left: 'auto',
            right: '0'
          })
        });
      });
      it('should not change transition beyond given rect', function () {
        var _utils10 = utils,
            drag = _utils10.drag,
            getByTestId = _utils10.getByTestId;
        var targetElement = getByTestId('main');
        var rect = targetElement.getBoundingClientRect();
        var startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        var delta = {
          x: 5,
          y: 5
        };
        drag({
          start: startAt,
          delta: delta
        });
        (0, _chai.expect)(targetElement.style.transform).to.include("translate(0px, 5px)");
      });
      it('should leave transition as it was before limit', function () {
        var _utils11 = utils,
            getByTestId = _utils11.getByTestId,
            beginDrag = _utils11.beginDrag,
            move = _utils11.move;
        var targetElement = getByTestId('main');
        targetElement.style.right = '50px';
        var rect = targetElement.getBoundingClientRect();
        var startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        var delta = {
          x: 5,
          y: 1
        };
        beginDrag(startAt);
        move({
          clientX: startAt.clientX + delta.x,
          clientY: startAt.clientY + delta.y
        });
        move({
          clientX: startAt.clientX + delta.x + 10,
          clientY: startAt.clientY + delta.y
        });
        move({
          clientX: startAt.clientX + delta.x + 25,
          clientY: startAt.clientY + delta.y
        });
        move({
          clientX: startAt.clientX + delta.x + 50,
          clientY: startAt.clientY + delta.y
        });

        var _targetElement$getBou = targetElement.getBoundingClientRect(),
            left = _targetElement$getBou.left,
            width = _targetElement$getBou.width;

        (0, _chai.expect)(left).to.equal(window.innerWidth - width);
      });
      it('should keep limits when dragging more than once', function () {
        var _utils12 = utils,
            drag = _utils12.drag,
            getByTestId = _utils12.getByTestId;
        var targetElement = getByTestId('main');
        targetElement.style.right = '50px';
        var rect = targetElement.getBoundingClientRect();
        var startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        var delta = {
          x: 15,
          y: 1
        };
        drag({
          start: startAt,
          delta: delta
        });
        drag({
          start: {
            clientX: startAt.clientX + delta.x,
            clientY: startAt.clientY + delta.y
          },
          delta: {
            x: 50,
            y: 0
          }
        });

        var _targetElement$getBou2 = targetElement.getBoundingClientRect(),
            left = _targetElement$getBou2.left,
            width = _targetElement$getBou2.width;

        (0, _chai.expect)(left).to.equal(window.innerWidth - width);
      });
    });
    describe('limit in rect', function () {
      var limits = {
        left: 11,
        right: window.innerWidth - 11,
        top: 5,
        bottom: window.innerHeight - 13
      };
      beforeEach(function () {
        (0, _react2.cleanup)();
        utils = setup({
          controlStyle: true,
          rectLimits: limits,
          style: _objectSpread({}, defaultStyle, {
            width: '180px',
            left: '20px'
          })
        });
      });
      it('should not change transition beyond given rect', function () {
        var _utils13 = utils,
            drag = _utils13.drag,
            getByTestId = _utils13.getByTestId;
        var targetElement = getByTestId('main');
        var rect = targetElement.getBoundingClientRect();
        var startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        var delta = {
          x: -50,
          y: -90
        };
        drag({
          start: startAt,
          delta: delta
        });
        (0, _chai.expect)(targetElement.getBoundingClientRect()).to.include({
          left: limits.left,
          top: limits.top
        });
      });
      it('should keep limits when dragging more than once', function () {
        var _utils14 = utils,
            drag = _utils14.drag,
            getByTestId = _utils14.getByTestId;
        var targetElement = getByTestId('main');
        targetElement.style.right = '50px';
        targetElement.style.left = 'auto';
        var rect = targetElement.getBoundingClientRect();
        var startAt = {
          clientX: rect.left + 5,
          clientY: rect.top + 5
        };
        var delta = {
          x: 15,
          y: 1
        };
        drag({
          start: startAt,
          delta: delta
        });
        drag({
          start: {
            clientX: startAt.clientX + delta.x,
            clientY: startAt.clientY + delta.y
          },
          delta: {
            x: 50,
            y: 0
          }
        });

        var _targetElement$getBou3 = targetElement.getBoundingClientRect(),
            left = _targetElement$getBou3.left,
            width = _targetElement$getBou3.width;

        (0, _chai.expect)(left).to.equal(limits.right - width);
      });
    });
  });

  function Consumer(props) {
    var _useDraggable = (0, _Draggable.useDraggable)(props),
        targetRef = _useDraggable.targetRef,
        handleRef = _useDraggable.handleRef,
        getTargetProps = _useDraggable.getTargetProps,
        delta = _useDraggable.delta,
        dragging = _useDraggable.dragging;

    var _props$style = props.style,
        style = _props$style === void 0 ? defaultStyle : _props$style;
    return _react.default.createElement("main", _extends({
      className: "container",
      ref: targetRef,
      "data-testid": "main",
      style: style
    }, getTargetProps()), dragging && _react.default.createElement("span", null, "Dragging to:"), _react.default.createElement("output", null, delta.x, ", ", delta.y), _react.default.createElement("button", {
      className: "handle",
      ref: handleRef
    }, "handle"));
  }

  function setup() {
    var props = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};
    var getters = (0, _react2.render)(_react.default.createElement(Consumer, props));

    function drag() {
      var _ref = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          _ref$start = _ref.start,
          start = _ref$start === void 0 ? {
        clientX: 0,
        clientY: 0
      } : _ref$start,
          _ref$delta = _ref.delta,
          delta = _ref$delta === void 0 ? {
        x: 0,
        y: 0
      } : _ref$delta,
          _ref$touch = _ref.touch,
          touch = _ref$touch === void 0 ? false : _ref$touch;

      beginDrag(start, touch);
      move({
        clientX: start.clientX + delta.x,
        clientY: start.clientY + delta.y
      }, touch);
      endDrag({
        clientX: start.clientX + delta.x,
        clientY: start.clientY + delta.y
      }, touch);
    }

    function beginDrag(start) {
      var touch = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : false;
      var target = getters.getByText(/handle/);

      if (touch) {
        _react2.fireEvent.touchStart(target, {
          touches: [createTouch(_objectSpread({
            target: target
          }, start))]
        });
      } else {
        _react2.fireEvent.mouseDown(target, start);
      }
    }

    function move(to, touch) {
      var target = getters.getByText(/handle/);

      if (touch) {
        _react2.fireEvent.touchMove(target, {
          touches: [createTouch(_objectSpread({
            target: target
          }, to))]
        });
      } else {
        _react2.fireEvent.mouseMove(target, to);
      }
    }

    function endDrag(end, touch) {
      var target = getters.getByText(/handle/);

      if (touch) {
        _react2.fireEvent.touchEnd(getters.getByText(/handle/), {
          changedTouches: [createTouch(_objectSpread({
            target: target
          }, end))]
        });
      } else {
        _react2.fireEvent.mouseUp(getters.getByText(/handle/), end);
      }
    }

    return _objectSpread({}, getters, {
      beginDrag: beginDrag,
      move: move,
      drag: drag
    });
  }
});

function createTouch(_ref2) {
  var target = _ref2.target,
      rest = _objectWithoutProperties(_ref2, ["target"]);

  return new Touch(_objectSpread({
    identifier: Date.now(),
    target: target
  }, rest));
}